from setuptools import setup
setup(
        name = 'pyshaun',
        packages = ['pyshaun'],
        version = '0.1.1',
        description = 'SHAUN implementation',
        author = 'Arthur Blanleuil',
        author_email = 'arthur.blanleuil@gmail.com',
        url = 'https://github.com/ablanleuil/pyshaun',
        download_url = 'https://github.com/ablanleuil/pyshaun/archive/0.0.1.tar.gz',
        keywords = ['notation language', 'library'],
        classifiers = []
)

